
package taxman;

import  java.awt.*;

/**
* This is the lower left button panel for the applet.
*  It controls the new game and options buttons and
*  supplies a checkbox for the user to choose to go first.
*
*  @author Jacob Crossman
*  @version 1.0
*/
public class CPUUpdatePanel extends Panel implements GameUpdate
{
   private static final Font LABEL_FONT = new Font("Helvetica", Font.BOLD, 12);
   private static final int  INTERVAL   = 50;                                //ms for update

   //This set of four labels are text for the statistics
   private Label    txt_think  = new Label("Thinking: ");
   private Label    txt_levels = new Label("Max Depth: ");
   private Label    txt_search = new Label("Moves Tried: ");
   private Label    txt_cutoff = new Label("Cutoffs: ");

   //These four labels are the numeric values for their respective
   // text labels
   private Label    num_think  = new Label(" XXXXXXXX ");
   private Label    num_levels = new Label(" XXXXXXXX ");
   private Label    num_search = new Label(" XXXXXXXX ");
   private Label    num_cutoff = new Label(" XXXXXXXX ");

   private long     start_time = 0;
   private boolean  cpu_move   = false;
   private Minimax  minimax    = null;

   /**
   * Initializes UI components and sets the layout.
   */
   public CPUUpdatePanel()
   {

      //Set up font for regular labels
      txt_think.setFont(LABEL_FONT);
      txt_think.setAlignment(Label.RIGHT);
      txt_levels.setFont(LABEL_FONT);
      txt_levels.setAlignment(Label.RIGHT);
      txt_search.setFont(LABEL_FONT);
      txt_search.setAlignment(Label.RIGHT);
      txt_cutoff.setFont(LABEL_FONT);
      txt_cutoff.setAlignment(Label.RIGHT);

      //Set up font for dynamic, numeric labels
      num_think.setFont(LABEL_FONT);
      num_think.setAlignment(Label.CENTER);
      num_levels.setFont(LABEL_FONT);
      num_levels.setAlignment(Label.CENTER);
      num_search.setFont(LABEL_FONT);
      num_search.setAlignment(Label.CENTER);
      num_cutoff.setFont(LABEL_FONT);
      num_cutoff.setAlignment(Label.CENTER);

      // Make it look good
      createLayout();
   }

   /**
   * Does the layout fancy work for the applet.
   *  It uses a gridBagLayout to create its particular look.
   */
   private void createLayout()
   {
      GridBagLayout      gbl = new GridBagLayout();
      GridBagConstraints gbc = new GridBagConstraints();

      setLayout(gbl);


      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(20,5,5,0);

      //Places "Thinking: " label on top
      gbc.anchor = GridBagConstraints.EAST;
      gbc.fill   = GridBagConstraints.HORIZONTAL;
      gbc.gridx  = 1;
      gbc.gridy  = 1;
      gbc.weightx = 1.0;
      gbc.weighty = 1.0;
      gbl.setConstraints(txt_think, gbc);
      add(txt_think);


      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(20,0,5,5);

      //Places Number of seconds dynamic panel
      gbc.anchor = GridBagConstraints.EAST;
      gbc.fill   = GridBagConstraints.HORIZONTAL;
      gbc.gridx  = 2;
      gbc.gridy  = 1;
      gbc.weightx = 1.0;
      gbc.weighty = 1.0;
      gbl.setConstraints(num_think, gbc);
      add(num_think);


      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(5,5,5,0);

      //Places "Max Depth: " on second line
      gbc.anchor = GridBagConstraints.EAST;
      gbc.fill   = GridBagConstraints.HORIZONTAL;
      gbc.gridx  = 1;
      gbc.gridy  = 2;
      gbc.weightx = 1.0;
      gbc.weighty = 1.0;
      gbl.setConstraints(txt_levels, gbc);
      add(txt_levels);


      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(5,0,5,5);

      //Places actual depth of search
      gbc.anchor = GridBagConstraints.EAST;
      gbc.fill   = GridBagConstraints.HORIZONTAL;
      gbc.gridx  = 2;
      gbc.gridy  = 2;
      gbc.weightx = 1.0;
      gbc.weighty = 1.0;
      gbl.setConstraints(num_levels, gbc);
      add(num_levels);


      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(5,5,5,0);

      //Places "Moves Tried: " label on third line
      gbc.anchor = GridBagConstraints.EAST;
      gbc.fill   = GridBagConstraints.HORIZONTAL;
      gbc.gridx  = 1;
      gbc.gridy  = 3;
      gbc.weightx = 1.0;
      gbc.weighty = 1.0;
      gbl.setConstraints(txt_search, gbc);
      add(txt_search);


      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(5,0,5,5);

      //Places actual number of moves tried in dynamic label
      gbc.anchor = GridBagConstraints.EAST;
      gbc.fill   = GridBagConstraints.HORIZONTAL;
      gbc.gridx  = 2;
      gbc.gridy  = 3;
      gbc.weightx = 1.0;
      gbc.weighty = 1.0;
      gbl.setConstraints(num_search, gbc);
      add(num_search);


      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(5,5,5,0);

      //Places "A/B Cutoffs: " on last row
      gbc.anchor = GridBagConstraints.EAST;
      gbc.fill   = GridBagConstraints.HORIZONTAL;
      gbc.gridx  = 1;
      gbc.gridy  = 4;
      gbc.weightx = 1.0;
      gbc.weighty = 1.0;
      gbl.setConstraints(txt_cutoff, gbc);
      add(txt_cutoff);


      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(5,0,5,5);

      //Places actual number of cutoffs at bottom
      gbc.anchor = GridBagConstraints.EAST;
      gbc.fill   = GridBagConstraints.HORIZONTAL;
      gbc.gridx  = 2;
      gbc.gridy  = 4;
      gbc.weightx = 1.0;
      gbc.weighty = 1.0;
      gbl.setConstraints(num_cutoff, gbc);
      add(num_cutoff);

      validate();
   }

   /**
   * Over-rides setBackground to set the internal components
   *  backgrounds to match
   * @param clr The color to set the background to
   */
   public void setBackground(Color clr)
   {
      super.setBackground(clr);

      //Set all sub-components
      txt_think.setBackground(clr);
      txt_levels.setBackground(clr);
      txt_search.setBackground(clr);
      txt_cutoff.setBackground(clr);

      //Set up reverse video for numeric labels
      num_think.setBackground(getForeground());
      num_levels.setBackground(getForeground());
      num_search.setBackground(getForeground());
      num_cutoff.setBackground(getForeground());

      //Set up reverse video for numeric labels
      num_think.setForeground(getBackground());
      num_levels.setForeground(getBackground());
      num_search.setForeground(getBackground());
      num_cutoff.setForeground(getBackground());
   }

   /**
   * Over-rides setForeground to set the internal components
   *  foregrounds to match
   * @param clr The color to set the foreground to
   */
   public void setForeground(Color clr)
   {
      super.setForeground(clr);

      //Set all sub-components
      txt_think.setForeground(clr);
      txt_levels.setForeground(clr);
      txt_search.setForeground(clr);
      txt_cutoff.setForeground(clr);

      //Set up reverse video for numeric labels
      num_think.setBackground(getForeground());
      num_levels.setBackground(getForeground());
      num_search.setBackground(getForeground());
      num_cutoff.setBackground(getForeground());

      //Set up reverse video for numeric labels
      num_think.setForeground(getBackground());
      num_levels.setForeground(getBackground());
      num_search.setForeground(getBackground());
      num_cutoff.setForeground(getBackground());

   }

   /**
   * The standard update method from GameUpdate
   * This one handles CPU_MOVE, NEW_GAME.
   * @param b The current game board
   * @param m The current minimax object
   * @param flags Indicator of current state
   */
   public void gameUpdate(Board b, Minimax m, int flags)
   {
      if(flags == Game.CPU_MOVE)
         startTrackingCPU(m);
      else if(flags == Game.NEW_GAME)
      {
         stopTrackingCPU();
         resetNumbers();
      }
      else if(flags == Game.HUMAN_MOVE || flags == Game.GAME_OVER)
         stopTrackingCPU();
   }

   /**
   * Starts tracking the CPU's thought (search process)
   * @param m The minimax algorithm to use to collect statistics
   */
   private void startTrackingCPU(Minimax m)
   {
       //Initialize timer and set flag for tracking
       // to start
       start_time = System.currentTimeMillis();
       cpu_move   = true;

       //Set "Thinking" text visible to track time
       txt_think.setForeground(getForeground());
       num_think.setBackground(getForeground());

       //Set minimax alias
       minimax = m;

       //Start screen updates
       updateNumbers();
   }

   /**
   * Stops the statistic gathering and goes back to idle state.
   *  The "Thinking" text disappears
   */
   private void stopTrackingCPU()
   {
      //Turnoff automatic redraws
      cpu_move = false;

      //Make Thinking text invisible
      txt_think.setForeground(getBackground());
      num_think.setBackground(getBackground());

      //Update the display
      repaint();
   }

   /**
   * Resets all numbers to 0 and set state to idle
   */
   private void resetNumbers()
   {
      //Make sure we stop timed updates
      stopTrackingCPU();

      //All numbers go back to 0
      num_think.setText("0");
      num_levels.setText("0");
      num_search.setText("0");
      num_cutoff.setText("0");
   }

   /**
   * Over-rides the standard update function
   *  This paints the screen and updates the diplay numbers
   * @param g The graphics context on which to write
   */
   public  void update(Graphics g)
   {
      //super.update(g);
      //paint(g);
      paintAll(g);

      //Will do a repaint if currently
      // cpu is thinking
      updateNumbers();
   }

   /**
   * Updates the numbers on screen by grabbing
   *  statistics from minimax
   */
   private final void updateNumbers()
   {
      //Only update if it is currently the cpu move
      if(cpu_move && minimax != null)
      {
         //Grab important stats from mimimax
         long search = minimax.getEvaluations();
         long cutoff = minimax.getCutoffs();
         long depth  = minimax.getMaxDepth();

         //Display stats in appropriate labels (including time)
         num_think.setText(Double.toString(((double)System.currentTimeMillis() - start_time) / 1000) + "s");
         num_levels.setText(Long.toString(depth));
         num_search.setText(Long.toString(search));
         num_cutoff.setText(Long.toString(cutoff));

         //Get set for timed repaint
         repaint(INTERVAL);
      }
   }

}
