
package taxman;

import java.awt.*;

/**	
* This little canvas show the winning or losing screen for
*  the game.
*
* @Author:	Jacob Crossman
* @Version:	1
*/
public class EndCanvas extends Canvas implements GameUpdate
{
   //Static constants for use in display
   private static final Font fnt        = new Font("TimesRoman", Font.BOLD, 25);
   private static final int  TIE        = 0;
   private static final int  HUMAN      = 1;
   private static final int  CPU        = 2;

   private static final String strWon1  = "YOU WON!!";
   private static final String strWon2  = "CONGRATULATIONS!";
   private static final String strLost1 = "YOU LOST!!";
   private static final String strLost2 = "YOU CAN NEVER BEAT THE TAXMAN!";
   private static final String strTie1  = "IT'S A TIE!!";
   private static final String strTie2  = "Nice Try. Why not play again?";

   private static final int    IMG_OFFSET = 35;

   private Image    imgWin   = null;          //winning image
   private Image    imgLoose = null;          //loosing image
   private Image    imgTie   = null;          //Tie image

   private int      winner   = TIE;           //start with a tie

   /**
   * Creates a canvas on which to display the winning
   * or loosing message.
   * @param win The image to display if the human wins
   * @param loose The image to display if the human looses
   * @param tie The image to display if there is a tie
   */
   public EndCanvas(Image win, Image loose, Image tie)
   {
      imgWin   = win;
      imgLoose = loose;
      imgTie   = tie;
   }

   /**
   * This is the common update method for all GameUpdate objects.
   *  It's job is to keep the panel synchronized with game state.
   * @param b The current game board
   * @param m The current minimax object
   * @param flag a flag indicating the game state
   */
   public void gameUpdate(Board b, Minimax m, int flags)
   {
       if(flags == Game.GAME_OVER)
       {
          if(b.getCPUScore() == b.getHumanScore())
             winner = TIE;
          else if(b.getWinner() == Board.HUMAN)
             winner = HUMAN;
          else
             winner = CPU;
       }

       repaint();
   }

   /**
   * Paints the final string and image on the canvas (centered)
   * @param g the graphics context on which to draw
   */
   public void paint(Graphics g)
   {
      //Need to set a large font
      g.setFont(fnt);

      //Set of variables useful for centering the image
      FontMetrics fm         = g.getFontMetrics();
      Rectangle   r          = bounds();
      int         half_w     = Math.round(r.width / 2);
      int         half_h     = Math.round(r.height / 2);
      int         char_h     = fm.getAscent();
      int         img_hw     = 0;

      if(winner == TIE)
      {
         //Draw "It's a Tie"
         g.drawString(strTie1,
                      half_w - fm.stringWidth(strTie1) / 2,
                      char_h);
         //Draw "Congratulations"
         g.drawString(strTie2,
                      half_w - fm.stringWidth(strTie2) / 2,
                      char_h + fm.getAscent());

         //Calculate the image half-width
         img_hw       = imgTie.getWidth(this) / 2;

         //Draw winning image
         g.drawImage(imgTie, half_w - img_hw, char_h + IMG_OFFSET, imgTie.getWidth(this), imgTie.getHeight(this),
                     getBackground(), this);
      }
      else if(winner == HUMAN)   //Human won
      {
         //Draw "You Won"
         g.drawString(strWon1,
                      half_w - fm.stringWidth(strWon1) / 2,
                      char_h);
         //Draw "Congratulations"
         g.drawString(strWon2,
                      half_w - fm.stringWidth(strWon2) / 2,
                      char_h + fm.getAscent());

         //Calculate the image half-width
         img_hw       = imgWin.getWidth(this) / 2;

         //Draw winning image
         g.drawImage(imgWin, half_w - img_hw, char_h + IMG_OFFSET, imgWin.getWidth(this), imgWin.getHeight(this),
                     getBackground(), this);
      }
      else
      {
         //Draw "You Lost"
         g.drawString(strLost1,
                      half_w - fm.stringWidth(strLost1) / 2,
                      char_h);
         //Draw an insult
         g.drawString(strLost2,
                      half_w - fm.stringWidth(strLost2) / 2,
                      char_h + fm.getAscent());

         //Calculate image half-width
         img_hw       = imgLoose.getWidth(this) / 2;

         //Draw the loosing image
         g.drawImage(imgLoose, half_w - img_hw, char_h + IMG_OFFSET, imgLoose.getWidth(this), imgLoose.getHeight(this),
                     getBackground(), this);
      }
   }

}

