package taxman;

import java.awt.*;
import java.util.*;

/**
* This is the table for gathering numbers for either side.
*  It can be for the human or for the CPU by changing the
*  constructor argument from Game.CPU_CHOSE to Game.HUMAN_CHOSE
*
* @author Jacob Crossman
* @version 1.0
*/
public class GatherTable extends GameTable
{
   public static final Color DEF_BACK_CHOSE  = DEF_BACK_COLOR;
   public static final Color DEF_BACK_TAX    = Color.red;

   private Image     backImgChose    = null;                //Background image for numbers you chose
   private Image     backImgTax      = null;                //Background image fore
   private Color     numClr          = DEF_NUM_COLOR;
   private Color     backClrChose    = DEF_BACK_CHOSE;
   private Color     backClrTax      = DEF_BACK_TAX;

   private int       side            = 0;                   //A constant determining the side we will track

   /**
   * The constructor takes many parameters and builds up a table which
   *  can respond to updates
   * @param num_color The color of the numbers that are displayed
   * @param bgChose The background image for numbers you chose (can be null)
   * @param bgTax The background image for numbers you collect as taxes (can be null)
   * @param track_cpu True if this grid should track the cpu, otherwise it tracks the human
   */
   public GatherTable(Color num_color, Image bgChose, Image bgTax, boolean track_cpu)
   {
      super();                      //Call base constructor to do some initial stuff

      numClr       = num_color;     //Set some look properties
      backImgChose = bgChose;
      backImgTax   = bgTax;

      side = (track_cpu)? Game.CPU_CHOSE: Game.HUMAN_CHOSE;
   }

   /**
   * The standard update method from GameUpdate
   * This one handles NEW_GAME, CPU_CHOSE, HUMAN_CHOSE.
   * @param b The current game board
   * @param m The current minimax object
   * @param flags Indicator of current state
   */
   public void gameUpdate(Board b, Minimax m, int flags)
   {
      super.gameUpdate(b,m,flags);                   //base class does some stuff

      if(flags == Game.CPU_CHOSE)
      {
         if(side == flags)
            addNumbersChosen(b);
         else
            addTaxesCollected(b);
      }
      else if(flags == Game.HUMAN_CHOSE)
      {
         if(side == flags)
            addNumbersChosen(b);
         else
            addTaxesCollected(b);
      }
   }

   /**
   * Adds the number that was chosen in the last move to this table.
   * @param b The current game board (can grab the last move from it)
   */
   private void addNumbersChosen(Board b)
   {
      //Add the number chosen
      Move last_move = b.getLastMove();

      if(backImgChose != null)
         addNumber(new GUINumber(last_move.scoreMade(), numClr, backImgChose));
      else
         addNumber(new GUINumber(last_move.scoreMade(), numClr, backClrChose));
   }

   /**
   * Adds the taxes collected in the last move to this grid.
   * @param b The current game board (can grab the last move from it)
   */
   private void addTaxesCollected(Board b)
   {
      Move last_move = b.getLastMove();

      //Add all the 'taxes' (or factors)
      Enumeration e = (last_move.getTaxList()).elements();
      while(e.hasMoreElements())
      {
         int tmpNum = ((TaxNumber)e.nextElement()).getValue();

         if(backImgTax != null)
            addNumber(new GUINumber(tmpNum, numClr, backImgTax));
         else
            addNumber(new GUINumber(tmpNum, numClr, backClrTax));
      }
   }
}

