
package taxman;

import  java.awt.*;

/**
* This is the lower left button panel for the applet.
*  It controls the new game and options buttons and
*  supplies a Checkbox for the user to choose to go first.
*
*  @author Jacob Crossman
*  @version 1.0
*/
public class LoadPanel extends Panel
{
   private static final Font   TITLE_FONT = new Font("Helvetica", Font.BOLD, 24);
   private static final Font   LABEL_FONT = new Font("Helvetica", Font.BOLD, 18);
   private static final Insets PROGRESS_INSETS = new Insets(4, 0, 4, 0);

   //Static field labels and the title of the options screen
   private Label     labTitle   = new Label("Taxman Media Files Loading, Please Be Patient");
   private Label     labFile    = new Label("Last File Loaded: ");
   private Label     labFName   = new Label("XXXXX None XXXXX");
   private Label     labPercent = new Label("Percentage Complete");

   private ProgressBar progress = null;
   private int         max      = 0;
   private int         current  = 0;
   
   /**
   * Initializes UI components and sets the layout.
   * @param NumMedia the number of media files that will be loaded
   */
   public LoadPanel(int NumMedia)
   {
      max = NumMedia;

      initializeObjects();
      createLayout();
   }

   private void initializeObjects()
   {
      //Create a new progress bar
      progress = new ProgressBar(0, 100, true);
      progress.resize(100, 30);
      progress.setFont(LABEL_FONT);
      progress.setInsets(PROGRESS_INSETS);

      //Set up font for title
      labTitle.setFont(TITLE_FONT);
      labTitle.setAlignment(Label.CENTER);

      //All the field labels have same font and
      // alignment
      labFile.setFont(LABEL_FONT);
      labFile.setAlignment(Label.RIGHT);
      labFName.setFont(LABEL_FONT);
      labFName.setAlignment(Label.LEFT);
      labPercent.setFont(LABEL_FONT);
      labPercent.setAlignment(Label.RIGHT);
   }

   /**
   * Does the layout fancy work for the applet.
   *  It uses a gridBagLayout to create its particular look.
   */
   private void createLayout()
   {
      Panel              ctr = new Panel();

      GridBagLayout      gbl = new GridBagLayout();
      GridBagConstraints gbc = new GridBagConstraints();

      ctr.setLayout(gbl);


      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(20,5,5,5);

      //Places TITLE on top
      gbc.anchor    = GridBagConstraints.CENTER;
      gbc.fill      = GridBagConstraints.HORIZONTAL;
      gbc.gridx     = 1;
      gbc.gridy     = 1;
      gbc.weightx   = 1.0;
      gbc.weighty   = 1.0;
      gbc.gridwidth = GridBagConstraints.REMAINDER;
      gbl.setConstraints(labTitle, gbc);
      ctr.add(labTitle);


      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(5,5,5,5);

      //Places Number of Search Levels second
      gbc.anchor    = GridBagConstraints.EAST;
      gbc.fill      = GridBagConstraints.HORIZONTAL;
      gbc.gridx     = 1;
      gbc.gridy     = 2;
      gbc.weightx   = 1.0;
      gbc.weighty   = 1.0;
      gbc.gridwidth = 1;
      gbl.setConstraints(labFile, gbc);
      ctr.add(labFile);


      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(5,5,5,5);

      //Places Search Levels modifiable field
      gbc.anchor    = GridBagConstraints.EAST;
      gbc.fill      = GridBagConstraints.HORIZONTAL;
      gbc.gridx     = 2;
      gbc.gridy     = 2;
      gbc.weightx   = 1.0;
      gbc.weighty   = 1.0;
      gbc.gridwidth = 1;
      gbl.setConstraints(labFName, gbc);
      ctr.add(labFName);


      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(5,5,5,5);

      //Places Quantity of Numbers tag
      gbc.anchor    = GridBagConstraints.EAST;
      gbc.fill      = GridBagConstraints.HORIZONTAL;
      gbc.gridx     = 1;
      gbc.gridy     = 3;
      gbc.weightx   = 1.0;
      gbc.weighty   = 1.0;
      gbc.gridwidth = 1;
      gbl.setConstraints(labPercent, gbc);
      ctr.add(labPercent);


      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(5,5,5,5);

      //Places Number of numbers update field
      gbc.anchor    = GridBagConstraints.EAST;
      gbc.fill      = GridBagConstraints.HORIZONTAL;
      gbc.gridx     = 2;
      gbc.gridy     = 3;
      gbc.weightx   = 1.0;
      gbc.weighty   = 1.0;
      gbc.gridwidth = 1;
      gbl.setConstraints(progress, gbc);
      ctr.add(progress);

      //Create main layout and put the layout with the labels
      // in the middle
      GridBagLayout      gbl3 = new GridBagLayout();
      setLayout(gbl3);

      //spacing for top, left, bottom, and right
      gbc.insets = new Insets(0,0,20,0);

      //Places All Options in the middle
      gbc.anchor    = GridBagConstraints.CENTER;
      gbc.fill      = GridBagConstraints.NONE;
      gbc.gridx     = 1;
      gbc.gridy     = 1;
      gbc.weightx   = 0.0;
      gbc.weighty   = 0.0;
      gbl3.setConstraints(ctr, gbc);
      add(ctr);

      validate();
   }

   /**
   * Over-ride of the inherited function to keep the
   *  text box and button text from changing color (they won't be clear)
   * @param clr The new foreground color
   */
   public void setBackground(Color clr)
   {
      super.setBackground(clr);

      //propagate background change to sub-components
      labTitle.setBackground(clr) ;
      labFile.setBackground(clr);
      labPercent.setBackground(clr);
   
      //Reverse video of file name and progress bar
      labFName.setBackground(getForeground());
      labFName.setForeground(getBackground());
      progress.setBackground(getForeground());
      progress.setForeground(getBackground());
   }

   /**
   * Over-ride of the inherited function to keep the
   *  text box and button text from changing color (they won't be clear)
   * @param clr The new background color
   */
   public void setForeground(Color clr)
   {
      super.setForeground(clr);


      //propagate background change to sub-components
      labTitle.setForeground(clr) ;
      labFile.setForeground(clr);
      labPercent.setForeground(clr);
   
      //Reverse video of file name and progress bar
      labFName.setBackground(getForeground());
      labFName.setForeground(getBackground());
      progress.setBackground(getForeground());
      progress.setForeground(getBackground());
   }

   /**
   * This method should be called every time a new file is
   * loaded.  It will update the loading screen and the progress bar.
   * @param Name the name of the file that was last loaded.
   */
   public void newFile(String Name)
   {
      //One more file loaded
      current++;

      //Change file that was loaded
      labFName.setText(Name);

      //Change progress bar and repaint screen
      progress.setValue(Math.round(100 *  (float)current / max));
      progress.repaint();
   }

}
