package taxman;

import java.util.*;

/**
* Class Move
*
* Defines a move in the game.
* Moves have a player who makes the move,
*  a move (TaxNumber) and taxes collected
*/
public class Move extends Object
{
   int       player;
   TaxNumber move;
   Vector    taxes;
   int       tax_value;

   /**
   * Creates a move out of a player, a number choice, and a vector of tax numbers.
   * @param Player The id for the player who made the move
   * @param Move The actuall move (number choice)
   * @param The list of available factors the taxman will take
   */
   public Move(int Player, TaxNumber Move, Vector Taxes)
   {
      player = Player;
      move   = Move;
      taxes  = Taxes;

      //Find the total value of the taxes (sum the factors)
      tax_value = 0;
      for(int i = 0; i < taxes.size(); i++)
         tax_value += ((TaxNumber)taxes.elementAt(i)).getValue();
   }

   /**
   * Returns the player Id for the player who made the move
   * @return The ID for the player who made the move
   */
   public final int getPlayer() { return player; }

   /**
   * Returns the score made by the mover.
   * @return The score made by the mover.
   */
   public final int scoreMade() { return move.getValue(); }

   /**
   * Returns the score made by the tax collector.
   * @return The score made by the tax collector.
   */
   public final int taxesCollected() { return tax_value; }

   /**
   * Returns the vector of taxes collected on this move
   * @return A vector with the factors (taxes) collected on this move
   */
   public final Vector getTaxList() { return taxes; }

   /**
   * Returns the move as a TaxNumber
   * @return A TaxNumber defining the move
   */
   public final TaxNumber getMove() { return move; }

   /**
   * Standard toString function.  It displays the move stats on one line
   * @return A string with the move stats
   */
   public final String toString()
   {
      StringBuffer tmp = new StringBuffer();

      //The Player ID and the move made
      tmp.append("P." + player + ": " + move.getValue() + " -- ");

      //Get all the vactors and print them
      Enumeration factors = taxes.elements();
      while(factors.hasMoreElements())
         tmp.append(((TaxNumber)factors.nextElement()).getValue() + ", ");

      return tmp.toString();
   }
}
