package taxman;

import java.awt.*;

/**
* This class updates the user with the current score and
*  game status.
*
* @author Jacob Crossman
* @version 1.0
*/
public class ScoreArea extends Panel implements GameUpdate
{
   private static final Font LABEL_FONT = new Font("Helvetica", Font.BOLD, 14);

   private Label human_score = new Label("XXXXX");          // The human's score
   private Label human_name  = new Label("YOU");            // 'YOU'

   private Label avail_text  = new Label("Numbers Left");   // Text for middle
   private Label avail_nums  = new Label("XXXX");           // Number of numbers remaining to choose

   private Label cpu_name    = new Label("CPU");            // 'CPU'
   private Label cpu_score   = new Label("XXXXX");          // CPU's Score


   /**
   * This is the constructor and it creates layouts and lays
   *  the panels/labels where they belong.
   */
   public ScoreArea()
   {
      //Set font and alignment for human score
      human_score.setFont(LABEL_FONT);
      human_score.setAlignment(Label.CENTER);
      human_name.setFont(LABEL_FONT);
      human_name.setAlignment(Label.LEFT);

      //Set font and alignment for the middle
      // free area counter
      avail_text.setFont(LABEL_FONT);
      avail_text.setAlignment(Label.CENTER);
      avail_nums.setFont(LABEL_FONT);
      avail_nums.setAlignment(Label.CENTER);

      //set font and alignment for the CPU score
      cpu_name.setFont(LABEL_FONT);
      cpu_name.setAlignment(Label.RIGHT);
      cpu_score.setFont(LABEL_FONT);
      cpu_score.setAlignment(Label.CENTER);

      //show reverse video for the scores
      human_score.setForeground(getBackground());
      avail_nums.setForeground(getBackground());
      cpu_score.setForeground(getBackground());

      human_score.setBackground(getForeground());
      avail_nums.setBackground(getForeground());
      cpu_score.setBackground(getForeground());

      createLayout();                             //do the tough work
   }

   /**
   * This method beautifies and lays out the scoreboard.
   */
   private void createLayout()
   {
      //We will use three grid bag layouts but only one
      // grid bag constraints object
      GridBagConstraints   gbc  = new GridBagConstraints();

      //These three sub-panels hold the text
      // so they can be layed out more appropriately
      Panel left   = new Panel();
      GridBagLayout        gbl1 = new GridBagLayout();

      left.setLayout(gbl1);

      //Pixel space top, WEST, bottom and EAST
      gbc.insets            = new Insets(2, 10, 2, 2);
      gbc.anchor            = GridBagConstraints.WEST;
      gbc.gridx             = 1;
      gbc.fill              = GridBagConstraints.HORIZONTAL;
      gbc.weightx           = 0.0;
      gbc.weighty           = 1.0;
      gbl1.setConstraints(human_score, gbc);
      left.add(human_score);


      //Pixel space top, WEST, bottom and EAST
      gbc.insets            = new Insets(2, 5, 2, 10);
      gbc.anchor            = GridBagConstraints.WEST;
      gbc.gridx             = 2;
      gbc.fill              = GridBagConstraints.HORIZONTAL;
      gbc.weightx           = 1.0;
      gbc.weighty           = 1.0;
      gbl1.setConstraints(human_name, gbc);
      left.add(human_name);

      ///////////////////////////////////////////////////
      // Work with the middle panel
      Panel middle       = new Panel();
      GridBagLayout gbl2 = new GridBagLayout();

      middle.setLayout(gbl2);

      //Pixel space top, WEST, bottom and EAST
      gbc.insets            = new Insets(2, 10, 2, 0);
      gbc.anchor            = GridBagConstraints.EAST;
      gbc.gridx             = 1;
      gbc.fill              = GridBagConstraints.HORIZONTAL;
      gbc.weightx           = 1.0;
      gbc.weighty           = 1.0;
      gbl2.setConstraints(avail_text, gbc);
      middle.add(avail_text);


      //Pixel space top, WEST, bottom and EAST
      gbc.insets            = new Insets(2, 0, 2, 20);
      gbc.anchor            = GridBagConstraints.WEST;
      gbc.gridx             = 2;
      gbc.fill              = GridBagConstraints.HORIZONTAL;
      gbc.weightx           = 0.25;
      gbc.weighty           = 1.0;
      gbl2.setConstraints(avail_nums, gbc);
      middle.add(avail_nums);


      ///////////////////////////////////////////////////
      // Work with the EAST side (CPU) panel
      Panel right        = new Panel();
      GridBagLayout gbl3 = new GridBagLayout();

      right.setLayout(gbl3);

      //Pixel space top, WEST, bottom and EAST
      gbc.insets            = new Insets(2, 10, 2, 5);
      gbc.anchor            = GridBagConstraints.EAST;
      gbc.gridx             = 1;
      gbc.fill              = GridBagConstraints.HORIZONTAL;
      gbc.weightx           = 1.0;
      gbc.weighty           = 1.0;
      gbl3.setConstraints(cpu_name, gbc);
      right.add(cpu_name);


      //Pixel space top, WEST, bottom and EAST
      gbc.insets            = new Insets(2, 2, 2, 10);
      gbc.anchor            = GridBagConstraints.EAST;
      gbc.gridx             = 2;
      gbc.fill              = GridBagConstraints.HORIZONTAL;
      gbc.weightx           = 0.0;
      gbc.weighty           = 1.0;
      gbl3.setConstraints(cpu_score, gbc);
      right.add(cpu_score);

      /////////////////////////////////////////////////
      //Finally, layout this panel with the sub-panels
      setLayout(new GridLayout(1,0));
      add(left);
      add(middle);
      add(right);

      validate();
   }

   /**
   * Over-rides the base function to allow reverse
   *  video for the scores and dynamic pieces of the
   *  score panel
   * @param clr The new foreground color
   */
   public void setForeground(Color clr)
   {
      super.setForeground(clr);

      //Ensure sub-component repaint
      human_name.setForeground(clr);
      avail_text.setForeground(clr);
      cpu_name.setForeground(clr);

      //show reverse video for the scores
      human_score.setForeground(getBackground());
      avail_nums.setForeground(getBackground());
      cpu_score.setForeground(getBackground());

      human_score.setBackground(getForeground());
      avail_nums.setBackground(getForeground());
      cpu_score.setBackground(getForeground());
   }

   /**
   * Over-rides the base function to allow reverse
   *  video for the scores and dynamic pieces of the
   *  score panel
   * @param clr The new background color
   */
   public void setBackground(Color clr)
   {
      super.setBackground(clr);

      //Ensure sub-component repaint
      human_name.setBackground(clr);
      avail_text.setBackground(clr);
      cpu_name.setBackground(clr);

      //show reverse video for the scores
      human_score.setForeground(getBackground());
      avail_nums.setForeground(getBackground());
      cpu_score.setForeground(getBackground());

      human_score.setBackground(getForeground());
      avail_nums.setBackground(getForeground());
      cpu_score.setBackground(getForeground());
   }

   /**
   * This is the common update method for all GameUpdate objects.
   *  It's job is to keep the panel synchronized with game state.
   * @param b The current game board
   * @param m The current minimax object
   * @param flag a flag indicating the game state
   */
   public void gameUpdate(Board b, Minimax m, int flags)
   {
       if(flags == Game.NEW_GAME)
          resetScores(b);
       else if(flags == Game.HUMAN_CHOSE || flags == Game.CPU_CHOSE)
          updateScores(b);

       repaint();
   }

   /**
   * Resets the scores shown onscreen to 0
   * @param b The board object for the new game
   */
   private void resetScores(Board b)
   {
      human_score.setText(" 0");
      cpu_score.setText("0 ");
      avail_nums.setText(Integer.toString(b.getNumberOfAvailableMoves()) + " ");
   }

   /**
   * Synchronizes the on-screen scores with internal game scores
   * @param b the current board object (with updated states)
   */
   private void updateScores(Board b)
   {
      human_score.setText(" " + Integer.toString(b.getHumanScore()));
      cpu_score.setText(Integer.toString(b.getCPUScore()) + " ");
      avail_nums.setText(Integer.toString(b.getNumberOfAvailableMoves()) + " ");
   }
}
