
package taxman;

import  java.awt.*;

/**
* This panel can swap the normal game view with the options view.
*
*  @author Jacob Crossman
*  @version 1.0
*/
public class SwapPanel extends Panel implements GameUpdate
{
   private static final String strOptions  = "Options";
   private static final String strGame     = "Game";
   private static final String strGameOver = "GameOver";

   private Panel     panOptions  = null;
   private Panel     panGame     = null;
   private Panel     panGameOver = null;

   private CardLayout layCard = new CardLayout();  //hold layout to allow swapping

   /**
   * Initializes UI components and sets the layout.
   * @param Game The game view panel
   * @param Options The options view panel
   */
   public SwapPanel(Panel Game, Panel Options, Panel GameOver)
   {
      panOptions  = Options;
      panGame     = Game;
      panGameOver = GameOver;

      // Make it look good
      createLayout();
   }

   /**
   * Does the layout fancy work for the applet.
   *  It uses CardLayout so that the options and game
   *  panels can be swapped automatically
   */
   private void createLayout()
   {
      //Set layout
      setLayout(layCard);

      //Add both panels
      add(strGame, panGame);
      add(strOptions, panOptions);
      add(strGameOver, panGameOver);

      //Update interface
      validate();
   }

   /**
   * The standard update method from GameUpdate
   * This one handles NEW_GAME, OPTIONS_OPEN and OPTIONS_CLOSE.
   * @param b The current game board
   * @param m The current minimax object
   * @param flags Indicator of current state
   */
   public void gameUpdate(Board b, Minimax m, int flags)
   {
      if(flags == Game.OPTIONS_OPEN)
         layCard.show(this, strOptions);
      else if(flags == Game.OPTIONS_CLOSE || flags == Game.NEW_GAME)
         layCard.show(this, strGame);
      else if(flags == Game.GAME_OVER)
         layCard.show(this, strGameOver);
   }
}
